"""Module to define Tmdb class."""
from datetime import datetime
from typing import Any
from typing import Dict
from typing import List
from typing import Optional

from aiohttp import ClientSession

from phylm.tools import initialize_tmdb_client


class Tmdb:
    """Class to abstract a TMDB result."""

    def __init__(
        self,
        raw_title: Optional[str] = None,
        movie_id: Optional[str] = None,
        raw_year: Optional[int] = None,
        api_key: Optional[str] = None,
        session: Optional[ClientSession] = None,
    ) -> None:
        """Initialize the object.

        Note that at least one of `raw_title` or `movie_id` must be given to be used as
        a search term. `movie_id` is preferred over `raw_title`.

        Args:
            raw_title: the title of the movie. Note that TMDB doesn't support fuzzy
                search.
            movie_id: the TMDB id of the movie.
            raw_year: an optional year for improved matching if only title is given.
            api_key: a TMDB api key. Must be supplied here or as an env var
            session: a `aiohttp.ClientSession` instance. One will be created if not
                supplied.

        Raises:
            ValueError: if neither `raw_title` nor `movie_id` is supplied.
        """
        if not (raw_title or movie_id):
            raise ValueError("At least one of raw_title and movie_id must be given")

        self.raw_title = raw_title
        self.movie_id = movie_id
        self.raw_year = raw_year
        self.low_confidence = False
        self.session = session
        self._api_key = api_key
        self._tmdb_data: Dict[str, Any] = {}

        self._client = initialize_tmdb_client(api_key, async_session=session)

    async def _get_tmdb_data(self) -> Dict[str, Any]:
        if self.movie_id:
            return await self._client.get_movie(self.movie_id)

        results = await self._client.search_movies_async(
            self.raw_title, year=self.raw_year  # type: ignore
        )

        if not results:
            return {}

        return await self._client.get_movie(results[0]["id"])

    async def load_source(self, session: Optional[ClientSession] = None) -> None:
        """Asynchronously load the data for from the source.

        Args:
            session: an optional `aiohttp.ClientSession` instance
        """
        if session:
            self._client = initialize_tmdb_client(self._api_key, async_session=session)

        self._tmdb_data = await self._get_tmdb_data()

    @property
    def title(self) -> Optional[str]:
        """Return the TMDB title.

        Returns:
            the title of the movie
        """
        return self._tmdb_data.get("title")

    @property
    def id(self) -> Optional[str]:
        """Return the TMDB id.

        Returns:
            the id of the movie
        """
        return str(self._tmdb_data.get("id"))

    @property
    def imdb_id(self) -> Optional[str]:
        """Return the IMDb id.

        Returns:
            the IMDb id of the movie
        """
        return str(self._tmdb_data.get("imdb_id"))

    def genres(self, limit: int = 3) -> List[str]:
        """Return the genres.

        Args:
            limit (int): an optional number of genres to return

        Returns:
            a list of the movie's genres
        """
        return [g["name"] for g in self._tmdb_data.get("genres", [])[:limit]]

    @property
    def runtime(self) -> Optional[int]:
        """Return the runtime.

        Returns:
            the runtime of the movie
        """
        runtime = self._tmdb_data.get("runtime")

        if not runtime:
            return None

        return int(runtime)

    @property
    def year(self) -> Optional[int]:
        """Return the movie's year.

        Returns:
            the year the movie was made
        """
        release_date = self._tmdb_data.get("release_date")

        if not release_date:
            return None

        date = datetime.strptime(release_date, "%Y-%m-%d")
        return int(date.year)

    @property
    def rating(self) -> Optional[float]:
        """Return the TMDB rating.

        Returns:
            the rating of the movie
        """
        rating = self._tmdb_data.get("vote_average")

        if not rating:
            return None

        return float(rating)

    @property
    def plot(self) -> Optional[str]:
        """Return the plot.

        Returns:
            the plot of the movie
        """
        return self._tmdb_data.get("overview")



"""Client to interact with The Movie DB (TMDB)."""
import os
from typing import Any
from typing import Dict
from typing import List
from typing import Optional
from typing import Union

from aiohttp import ClientSession
from requests import Session

from phylm.errors import NoTMDbApiKeyError


class TmdbClient:
    """Class to abstract to the Tmdb API."""

    def __init__(
        self, api_key: str, async_session: Optional[ClientSession] = None
    ) -> None:
        """Initialize the client.

        Args:
            api_key: an api_key for authentication
            async_session: an optional instance of `aiohttp.ClientSession`
        """
        self.session = Session()
        self.async_session = async_session or ClientSession()
        self.api_key = api_key
        self._base_url = "https://api.themoviedb.org/3"

    def search_movies(self, query: str, region: str = "us") -> List[Dict[str, Any]]:
        """Search for movies.

        Args:
            query: the search query
            region: the region for the query, affects the release date value

        Returns:
            List[Dict[str, Any]]: the search results
        """
        payload = {
            "api_key": self.api_key,
            "language": "en-US",
            "query": query,
            "include_adult": "false",
            "region": region.upper(),
        }
        res = self.session.get(f"{self._base_url}/search/movie", params=payload)

        res.raise_for_status()

        results: List[Dict[str, Any]] = res.json()["results"]
        return results

    async def search_movies_async(
        self, query: str, region: str = "us", year: Optional[int] = None
    ) -> List[Dict[str, Any]]:
        """Search for movies async.

        Args:
            query: the search query
            region: the region for the query, affects the release date value
            year: the year of the movie

        Returns:
            List[Dict[str, Any]]: the search results
        """
        params: Dict[str, Union[str, int]] = {
            "api_key": self.api_key,
            "language": "en-US",
            "query": query,
            "include_adult": "false",
            "region": region.upper(),
            "page": 1,
        }

        if year:
            params["year"] = year

        async with self.async_session.get(
            f"{self._base_url}/search/movie", params=params
        ) as resp:
            results = await resp.json()

        movies: List[Dict[str, Any]] = results["results"]
        return movies

    async def get_movie(self, movie_id: str) -> Dict[str, Any]:
        """Return a movie by id.

        Args:
            movie_id: the tmdb id of the movie

        Returns:
            Dict[str, Any]: a dictionary of the movie data
        """
        params = {
            "api_key": self.api_key,
            "language": "en-US",
        }

        async with self.async_session.get(
            f"{self._base_url}/movie/{movie_id}", params=params
        ) as resp:
            movie: Dict[str, Any] = await resp.json()

        return movie

    def get_streaming_providers(
        self, movie_id: str, regions: List[str]
    ) -> Dict[str, Any]:
        """Return a list of streaming providers for a given movie.

        Args:
            movie_id: the tmdb id of the movie
            regions: a list of regions to trim down the return list

        Returns:
            Dict[str, Any]: a dictionary of streaming providers, keyed by region name
        """
        payload = {"api_key": self.api_key}

        res = self.session.get(
            f"{self._base_url}/movie/{movie_id}/watch/providers", params=payload
        )

        res.raise_for_status()

        results: Dict[str, Any] = res.json()["results"]

        return {key: results.get(key.upper(), {}) for key in regions}


def initialize_tmdb_client(
    api_key: Optional[str] = None,
    async_session: Optional[ClientSession] = None,
) -> TmdbClient:
    """Initialize and return a TmdbClient.

    Args:
        api_key: an optional api_key to take precedence over an env var key
        async_session: an optional aiohttp ClienSession

    Raises:
        NoTMDbApiKeyError: when no api_key has been provided

    Returns:
        TmdbClient: an authorized Tmdb client
    """
    tmdb_api_key = api_key or os.environ.get("TMDB_API_KEY")

    if not tmdb_api_key:
        raise NoTMDbApiKeyError("An `api_key` must be provided to use this service")

    return TmdbClient(api_key=tmdb_api_key, async_session=async_session)



"""Module to hold `phylm` tools."""
from typing import Any
from typing import Dict
from typing import List
from typing import Optional
from typing import Union

from imdb.Movie import Movie

from phylm.clients.tmdb import initialize_tmdb_client
from phylm.sources.imdb import ia


def search_movies(query: str) -> List[Dict[str, Union[str, int]]]:
    """Return a list of search results for a query.

    Args:
        query: the search query

    Returns:
        a list of search results
    """
    results: List[Movie] = ia.search_movie(query)

    return [
        {
            "title": r.data.get("title"),
            "kind": r.data.get("kind"),
            "year": r.data.get("year"),
            "imdb_id": r.movieID,
            "cover_photo": r.data.get("cover url"),
        }
        for r in results
    ]


def search_tmdb_movies(
    query: str, api_key: Optional[str] = None, region: str = "us"
) -> List[Dict[str, Any]]:
    """Search for movies on TMDb.

    Args:
        query: the query string
        api_key: an api_key can either be provided here or through a TMDB_API_KEY env
            var
        region: an optional region to provide with the search request, affects the
            release_date value returned, must be provided in ISO 3166-1 format (eg.
            "us" or "gb")

    Returns:
        List[Dict[str, Any]]: the search results
    """
    client = initialize_tmdb_client(api_key=api_key)

    return client.search_movies(query=query, region=region)


def get_streaming_providers(
    tmdb_movie_id: str,
    regions: List[str],
    api_key: Optional[str] = None,
) -> Dict[str, Any]:
    """Return a list of streaming providers for a given movie.

    Args:
        tmdb_movie_id: the tmdb id of the movie
        regions: a list of regions to trim down the return list
        api_key: an api_key can either be provided here or through a TMDB_API_KEY env
            var

    Returns:
        Dict[str, Any]: a dictionary of streaming providers, keyed by region name
    """
    client = initialize_tmdb_client(api_key=api_key)

    return client.get_streaming_providers(movie_id=tmdb_movie_id, regions=regions)



"""Module to contain the IMDb class definition."""
import asyncio
from typing import List
from typing import Optional

import imdb
from imdb._exceptions import IMDbDataAccessError
from imdb.Movie import Movie

ia = imdb.Cinemagoer()


class Imdb:
    """Class to abstract an IMDb movie object."""

    def __init__(
        self,
        raw_title: Optional[str] = None,
        movie_id: Optional[str] = None,
        raw_year: Optional[int] = None,
    ) -> None:
        """Initialize the object.

        Note that at least one of `raw_title` or `movie_id` must be given to be used as
        a search term. `movie_id` is preferred over `raw_title`.

        Args:
            raw_title: the title of the movie
            movie_id: the `IMDb` id of the movie
            raw_year: an optional year for improved matching if only title is given

        Raises:
            ValueError: if neither `raw_title` nor `movie_id` is supplied
        """
        if not (raw_title or movie_id):
            raise ValueError("At least one of raw_title and movie_id must be given")

        self.raw_title = raw_title
        self.movie_id = movie_id
        self.raw_year = raw_year
        self.low_confidence = False
        self._imdb_data: Optional[Movie] = None

    def _get_imdb_data(self) -> Optional[Movie]:
        """Fetch the data from IMDb.

        If `self.movie_id` exists, prefer that as a search query, falling back to
        `self.raw_title` if that exists. If `movie_id` exists but is unrecognised
        by `IMDb` then we also fall back to the `raw_title`.

        Returns:
            an optional `IMDb` `Movie` object
        """
        if self.movie_id:
            try:
                get_movie_result: Movie = ia.get_movie(self.movie_id)
                if get_movie_result:
                    return get_movie_result
            except IMDbDataAccessError:
                pass

        if not self.raw_title:
            return None

        results: List[Movie] = [
            result
            for result in ia.search_movie(self.raw_title)
            if result.get("kind") == "movie"
        ]

        if not results:
            return None

        target = self._find_match(results)

        ia.update(target, info=["main"])

        return target

    def _find_match(self, results: List[Movie]) -> Optional[Movie]:
        """Find a match based on year or title.

        Args:
            results: A list of search results

        Returns:
            Optional[Movie]: the matched movie if found
        """
        # first try matching on year
        if self.raw_year:
            for result in results:
                if result.get("year") == self.raw_year:
                    return result

        # then try matching on title
        if self.raw_title:
            for result in results:
                if result["title"].lower() == self.raw_title.lower():
                    return result

        # finally pick the first result
        self.low_confidence = True
        return results[0]

    async def load_source(self) -> None:
        """Asynchronously load the data for from the source."""
        loop = asyncio.get_running_loop()
        self._imdb_data = await loop.run_in_executor(None, self._get_imdb_data)

    @property
    def title(self) -> Optional[str]:
        """Return the IMDb title.

        Returns:
            the title of the movie
        """
        if not self._imdb_data:
            return None

        return str(self._imdb_data.get("title"))

    @property
    def id(self) -> Optional[str]:
        """Return the IMDb id.

        Returns:
            the id of the movie
        """
        if not self._imdb_data:
            return None

        return str(self._imdb_data.movieID)

    def genres(self, limit: int = 3) -> List[str]:
        """Return the genres.

        Args:
            limit (int): an optional number of genres to return

        Returns:
            a list of the movie's genres
        """
        if not self._imdb_data:
            return []

        return list(self._imdb_data.get("genres", [])[:limit])

    def cast(self, limit: int = 5) -> List[str]:
        """Return the cast.

        Args:
            limit (int): an optional number of cast members to return

        Returns:
            a list of the movie's cast members
        """
        if not self._imdb_data:
            return []

        return [person["name"] for person in self._imdb_data.get("cast", [])[:limit]]

    @property
    def runtime(self) -> Optional[str]:
        """Return the runtime.

        Returns:
            the runtime of the movie
        """
        if not self._imdb_data:
            return None

        runtimes = self._imdb_data.get("runtimes")

        if runtimes:
            return str(runtimes[0])

        return None

    @property
    def year(self) -> Optional[int]:
        """Return the movie's year.

        Returns:
            the year the movie was made
        """
        if not self._imdb_data:
            return None

        year = self._imdb_data.get("year")

        if not year:
            return None

        return int(year)

    def directors(self, limit: int = 3) -> List[str]:
        """Return the director(s).

        Args:
            limit (int): an optional number of director to return

        Returns:
            a list of the movie's directors
        """
        if not self._imdb_data:
            return []

        return [
            person["name"] for person in self._imdb_data.get("directors", [])[:limit]
        ]

    @property
    def rating(self) -> Optional[float]:
        """Return the IMDb rating.

        Returns:
            the rating of the movie
        """
        if not self._imdb_data:
            return None

        rating = self._imdb_data.get("rating")

        if not rating:
            return None

        return float(rating)

    @property
    def plot(self) -> Optional[str]:
        """Return the plot.

        Returns:
            the plot of the movie
        """
        if not self._imdb_data:
            return None

        data = self._imdb_data
        if "plot" not in data.current_info:
            ia.update(data, info=["plot"])

        plot = data.get("plot")

        if not plot:
            return None

        return str(plot[0].split("::")[0])



"""Module to contain some web helper functions."""
from typing import Optional
from urllib.parse import quote_plus

import requests
from aiohttp import ClientSession
from bs4 import BeautifulSoup


# DEFAULT_HEADERS = {"User-agent": "Mozilla/5.0"}
DEFAULT_HEADERS = {
    "User-agent": (
        "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) "
        "AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/108.0.0.0 Safari/537.36"
    ),
}


def soupify(url: str) -> BeautifulSoup:
    """Get a webpage and return the BeautifulSoup representation.

    Args:
        url (str): the url for scraping

    Returns:
        a `BeautifulSoup` representation of the given url
    """
    search = requests.get(url, headers=DEFAULT_HEADERS).text
    return BeautifulSoup(search, "html.parser")


async def async_soupify(
    url: str, session: Optional[ClientSession] = None
) -> BeautifulSoup:
    """Asynchronously get a webpage and return the BeautifulSoup representation.

    Args:
        url: the url for scraping
        session: an optional instance of `aiohttp.ClientSession` in which to run the
            request. If a session is passed here then it will remain open after this
            function returns.

    Returns:
        a `BeautifulSoup` representation of the given url
    """
    keep_session = False
    if session:
        keep_session = True

    session = session or ClientSession()
    async with session.get(url, headers=DEFAULT_HEADERS) as resp:
        html = await resp.text()

    if not keep_session:
        await session.close()

    return BeautifulSoup(html, "html.parser")


def url_encode(string: str) -> str:
    """URL encode a string to be used in search queries.

    Args:
        string (str): the string to be url encoded

    Returns:
        a url encoded string
    """
    return quote_plus(string)



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['phylm', 'phylm.clients', 'phylm.sources', 'phylm.utils']

package_data = \
{'': ['*']}

install_requires = \
['aiohttp>=3.8.1,<4.0.0',
 'beautifulsoup4>=4.10.0,<5.0.0',
 'cinemagoer>=2022.12.27,<2023.0.0',
 'click>=8.0.1,<9.0.0',
 'requests>=2.26.0,<3.0.0']

entry_points = \
{'console_scripts': ['phylm = phylm.__main__:main']}

setup_kwargs = {
    'name': 'phylm',
    'version': '6.1.0',
    'description': 'Phylm',
    'long_description': '# Phylm\n\n[![Actions Status](https://github.com/dbatten5/phylm/workflows/Tests/badge.svg)](https://github.com/dbatten5/phylm/actions)\n[![Actions Status](https://github.com/dbatten5/phylm/workflows/Release/badge.svg)](https://github.com/dbatten5/phylm/actions)\n[![codecov](https://codecov.io/gh/dbatten5/phylm/branch/master/graph/badge.svg?token=P233M48EA6)](https://codecov.io/gh/dbatten5/phylm)\n[![PyPI version](https://badge.fury.io/py/phylm.svg)](https://badge.fury.io/py/phylm)\n\nFilm data aggregation with async support.\n\n## Motivation\n\nWhen deciding which film to watch next, it can be helpful to have some key datapoints at\nyour fingertips, for example, the genre, the cast, the Metacritic score and, perhaps\nmost importantly, the runtime. This package provides a Phylm class to gather information\nfrom various sources for a given film.\n\n## Installation\n\n```bash\npip install phylm\n```\n\n## Usage\n\n```python\n>>> import asyncio\n>>> from phylm import Phylm\n>>> p = Phylm("The Matrix")\n>>> asyncio.run(p.load_source("imdb"))\n>>> p.imdb.year\n1999\n>>> p.imdb.rating\n8.7\n```\n\n`phylm` also provides some tools around movie search results and more:\n\n```python\n>>> from phylm.tools import search_movies, get_streaming_providers\n>>> search_movies("the matrix")\n[{\n  \'title\': \'The Matrix\',\n  \'kind\': \'movie\',\n  \'year\': 1999,\n  \'cover_photo\': \'https://some-url.com\',\n  \'imdb_id\': \'0133093\',\n}, {\n  \'title\': \'The Matrix Reloaded\',\n  \'kind\': \'movie\',\n  \'year\': 2003,\n  \'cover_photo\': \'https://some-url.com\',\n  \'imdb_id\': \'0234215\',\n}, {\n...\n>>> get_streaming_providers("0234215", regions=["gb"])\n{\n  \'gb\': {\n    \'rent\': [{\n      \'display_priority\': 8,\n      \'logo_path\': \'/pZgeSWpfvD59x6sY6stT5c6uc2h.jpg\',\n      \'provider_id\': 130,\n      \'provider_name\': \'Sky Store\',\n    }],\n    \'flatrate\': [{\n      \'display_priority\': 8,\n      \'logo_path\': \'/ik9djlxNlex6sY6Kjsundc2h.jpg\',\n      \'provider_id\': 87,\n      \'provider_name\': \'Netflix\',\n    }]\n  }, {\n  ...\n}\n```\n\n## Help\n\nSee the [documentation](https://dbatten5.github.io/phylm) for more details.\n\n## Licence\n\nMIT\n',
    'author': 'Dom Batten',
    'author_email': 'dominic.batten@googlemail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/dbatten5/phylm',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8.1,<4.0.0',
}


setup(**setup_kwargs)



"""Module to hold the Rt class definition."""
from typing import Optional

from aiohttp import ClientSession
from bs4 import BeautifulSoup
from bs4.element import Tag

from phylm.utils.web import async_soupify
from phylm.utils.web import url_encode

RT_BASE_MOVIE_URL = "https://www.rottentomatoes.com/search"


class Rt:
    """Class to abstract a Rotten Tomatoes result."""

    def __init__(self, raw_title: str, raw_year: Optional[int] = None) -> None:
        """Initialize the object.

        Args:
            raw_title: the given title of the movie
            raw_year: an optional year for improved matching
        """
        self.raw_title = raw_title
        self.raw_year = raw_year
        self.low_confidence = False
        self._rt_data: Optional[Tag] = None

    def _parse_data(self, soup: BeautifulSoup) -> Optional[Tag]:
        results = soup.find_all("search-page-media-row")

        if not results:
            return None

        # first try matching on year
        for result in results:
            release_year = result["releaseyear"]
            if self.raw_year and str(self.raw_year) == release_year:
                return result

        # then try matching on title
        for result in results:
            result_title: str = result.find_all("a")[-1].string.strip()
            if result_title.lower() == self.raw_title.lower().strip():
                return result

        # finally pick the first result
        self.low_confidence = True
        return results[0]

    async def _scrape_data(
        self, session: Optional[ClientSession] = None
    ) -> BeautifulSoup:
        url_encoded_film = url_encode(self.raw_title)
        search_url = f"{RT_BASE_MOVIE_URL}?search={url_encoded_film}"
        return await async_soupify(search_url, session)

    async def load_source(self, session: Optional[ClientSession] = None) -> None:
        """Asynchronously load the data from the source.

        Args:
            session: an optional instance of `aiohttp.ClientSession` in which to run the
                request
        """
        raw_data = await self._scrape_data(session=session)
        self._rt_data = self._parse_data(raw_data)

    @property
    def title(self) -> Optional[str]:
        """Return the title.

        Returns:
            the title of the movie
        """
        if not self._rt_data:
            return None

        title_tag = self._rt_data.find_all("a")[-1]

        if title_tag:
            return str(title_tag.get_text()).strip()

        return None

    @property
    def year(self) -> Optional[str]:
        """Return the year.

        Returns:
            the year of the movie
        """
        if not self._rt_data:
            return None

        return str(self._rt_data["releaseyear"])

    @property
    def tomato_score(self) -> Optional[str]:
        """Return the Tomatometer Score.

        Returns:
            the tomatometer score
        """
        if not self._rt_data:
            return None

        return str(self._rt_data["tomatometerscore"])
